#include <stdio.h>
#include <math.h>

  /* Kvadratna jednadzba u obliku x^2 + p*x + q = 0.
     Pretpostavke (bez provjere): jednadzba ima realna rjesenja. */


  /* Rjesenja kvadratne jednadzbe obicnim i Vietinim formulama,
     u tipu double. */


    /* Funkcija za rjesavanje kvadratne jednadzbe u double. */

void kvad_double(double p, double q,
                 double *px_1, double *px_2,
                 double *pv_1, double *pv_2)
{
    double diskr, mp_half, x_1, x_2, v_1, v_2, temp;

    mp_half = -p / 2;
    diskr = sqrt(mp_half * mp_half - q);

        /* Obicne formule. */

    x_1 = mp_half + diskr;
    x_2 = mp_half - diskr;

        /* Za usporedbu, poredamo rastuce po apsolutnoj vrijednosti. */
    if (fabs(x_1) > fabs(x_2)) {
        temp = x_1;
        x_1 = x_2;
        x_2 = temp;
    }

        /* Prvo veci po apsolutnoj vrijednosti, drugi iz Vietine formule. */

    if (mp_half >= 0.0)
        v_2 = mp_half + diskr;
    else
        v_2 = mp_half - diskr;

        /* Provjera prije dijeljenja. Uociti: v_2 = 0 => v_1 = 0. */
    if (v_2 != 0.0)
        v_1 = q / v_2;
    else
        v_1 = 0.0;

        /* Vracanje kroz "varijabilne" argumente. */
    *px_1 = x_1;
    *px_2 = x_2;
    *pv_1 = v_1;
    *pv_2 = v_2;
}

    /* Funkcije za ispis koeficijenata i rjesenja. */

void print_coef(double p, double q)
{
    printf("   Egzaktni koeficijenti kvadratne jednadzbe"
           " x^2 + p*x + q = 0\n");
    printf("     p = %8.5f,  q = %8.5f\n", p, q);
    return;
}

void print_roots(double x_1, double x_2)
{
    printf("   Egzaktna rjesenja (iz tipa extended)\n");
    printf("     x_1 = %21.17f\n", x_1);
    printf("     x_2 = %21.17f\n", x_2);
    return;
}

void print_roots_double(double x_1, double x_2,
                        double x_1_exact, double x_2_exact)
{
    double rel_err_1 = 0.0, rel_err_2 = 0.0;

    if (x_1_exact != 0.0)
        rel_err_1 = (x_1_exact - x_1) / x_1_exact;
    if (x_2_exact != 0.0)
        rel_err_2 = (x_2_exact - x_2) / x_2_exact;

    printf("     x_1 = %21.17f,    rel_greska = %13.5e\n", x_1, rel_err_1);
    printf("     x_2 = %21.17f,    rel_greska = %13.5e\n", x_2, rel_err_2);
    return;
}

    /* Funkcija za obradu jedne kvadratne jednadzbe. */

void kvad_jedn(double p, double q,
               double x_1_exact, double x_2_exact)
{
    double x_1, x_2;    /* Izracunata rjesenja obicnim formulama. */
    double v_1, v_2;    /* Izracunata rjesenja Vietinim formulama. */

    print_coef(p, q);
    printf("\n");
    print_roots(x_1_exact, x_2_exact);    /* Egzaktna rjesenja. */

    printf("\n   ***** Double aritmetika\n");
    kvad_double(p, q, &x_1, &x_2, &v_1, &v_2);
    printf("\n   Obicne formule:\n");
    print_roots_double(x_1, x_2, x_1_exact, x_2_exact);
    printf("\n   Vietine formule:\n");
    print_roots_double(v_1, v_2, x_1_exact, x_2_exact);

    printf("\n");
    return;
}

    /* Glavni program. */

int main(void)
{
    double p, q;        /* Koeficijenti kvadratne jednadzbe. */
    double x_1, x_2;    /* Egzaktna rjesenja, izracunata u extended tipu. */


        /* Gautschijev primjer. */

    p = -56.0;
    q = 1.0;
    x_1 =  1.78628407335548633E-0002;
    x_2 =  5.59821371592664451E+0001;

    printf("\n");
    printf(" Gautschijev primjer:\n");
    printf(" ===================\n\n");
    kvad_jedn(p, q, x_1, x_2);


        /* Gautschijev primjer, modifikacija 1. */

    p = -49.0;
    q = 1.0;
    x_1 =  2.04166702127055996E-0002;
    x_2 =  4.89795833297872944E+0001;

    printf("\n");
    printf(" Gautschijev primjer, modifikacija 1:\n");
    printf(" ===================================\n\n");
    kvad_jedn(p, q, x_1, x_2);


        /* Gautschijev primjer, modifikacija 2. */

    p = -490.0;
    q = 1.0;
    x_1 =  2.04082482646116796E-0003;
    x_2 =  4.89997959175173539E+0002;

    printf("\n");
    printf(" Gautschijev primjer, modifikacija 2:\n");
    printf(" ===================================\n\n");
    kvad_jedn(p, q, x_1, x_2);


        /* Skoro dvostruka nultocka u 2. Diskriminanta blizu nule. */
        /* Ovaj primjer NE valja, jer se oduzimanje/kracenje u diskriminanti
           dogadja u double, pa se tek onda zaokruzuje.
           Ovdje su rezultati LOSI, jer racunanje p/2 NIJE egzaktno
                        p/2 = -2.00005,
             izracunati p/2 = -2.0001.
           Kvadriranje jos povecava ovu gresku, pa nakon KRACENJA u
           diskriminanti dobivamoe VELIKU relativnu gresku. */

    p = -4.0001 ;
    q = 4.0 ;
    x_1 =  1.98590777598819763E+0000;
    x_2 =  2.01419222401180237E+0000;

    printf("\n");
    printf(" Skoro dvostruka nultocka u 2:\n");
    printf(" ============================\n\n");
    kvad_jedn(p, q, x_1, x_2);


        /* Skoro dvostruka nultocka u 10. Diskriminanta blizu nule. */
        /* Ovaj primjer NE valja, jer se oduzimanje/kracenje u diskriminanti
           dogadja u double, pa se tek onda zaokruzuje.
           Ovdje su rezultati LOSI, jer racunanje p/2 NIJE egzaktno
                        p/2 = -10.0005,
             izracunati p/2 = -10.001.
           Kvadriranje jos povecava ovu gresku, pa nakon KRACENJA u
           diskriminanti dobivamoe VELIKU relativnu gresku. */

    p = -20.001 ;
    q = 100.0 ;
    x_1 =  9.90049875000781246E+0000;
    x_2 =  1.01005012499921875E+0001;

    printf("\n");
    printf(" Skoro dvostruka nultocka u 10:\n");
    printf(" =============================\n\n");
    kvad_jedn(p, q, x_1, x_2);

    return 0;
}
