  /* Zadatak:
       Naci broj svih prikazivih prostih brojeva, tj. broj prostih
       brojeva manjih ili jednakih INT_MAX (iz <limits.h>).
  */

  /* Koristimo trecu (najbrzu) varijantu funkcije za proste brojeve:
       - test djelitelja 2, a zatim samo neparnih do (int) sqrt(n).
     Sve petlje po djeliteljima su for petlje!

     Napomena: sve radimo u tipu int (ignoriramo brojeve <= 0).

     Dodana je stoperica (v. Prog2, zadnje predavanje) i ispis vremena.
  */

#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <time.h>


    /* Funkcija za procesorsko vrijeme u sekundama. */

double dsecnd (void) {
    return (double)( clock( ) ) / CLOCKS_PER_SEC;
}


    /* Najbrza varijanta funkcije za provjeru je li broj prost. */

int prost_3(int n)
{
        /* Prvo testira parnost (djelitelj 2), a onda testira sve
           neparne djelitelje od 2 do max_d = (int) sqrt(n). */

    int d;        /* Potencijalni djelitelj. */
    int max_d;    /* Maksimalni potencijalni djelitelj. */

        /* Brojevi <= 1 nisu prosti (ignoriramo <= 0). */
    if (n <= 1) return 0;


        /* Testiraj parnost broja. Ako je paran, samo 2 je prost. */
    if (n % 2 == 0) return (n == 2);

        /* Najveci djelitelj kojeg treba provjeriti je
             max_d = (int) sqrt(n).
           Naime, ako n nije prost, onda ima dva netrivijalna djelitelja,
           a manji od njih je najvise jednak max_d. Ako takav djelitelj od n
           ne postoji, onda je n prost. */
        /* Napomena: Zaokruzivanje sqrt(n) na donje cijelo je korektno
           na cijelom tipu za n, tj. ne moza se dogoditi da dobijemo za
           jedan premalo. Posebno, za brojeve n koji su potpun kvadrat,
           uvijek dobijemo korektan cjelobrojni korijen. */
    max_d = (int) sqrt(n);

        /* Petlja po mogucim neparnim djeliteljima od n od 3 do max_d. */
    for (d = 3; d <= max_d; d += 2)
        if (n % d == 0) return 0;    /* Nije prost. */

    return 1;
}


int main(void)
{
    int n, max_p, broj_p;
    double t1, t2, time;    /* Vremena za stopericu. */

    t1 = dsecnd( );    /* Pocetak mjerenja vremena. */

        /* Broj 2, kao jedini paran prost broj. */
    broj_p = 1;

        /* Dalje idemo po neparnim brojevima, pocev od 3.
           U petlji po neparnim brojevima idemo samo do INT_MAX - 2,
           zato da n ostane prikaziv NAKON sto dodamo 2.
           Ako stavimo max_p = INT_MAX, na kraju n postane negativan
           i dobijemo beskonacnu petlju! */
    max_p = INT_MAX - 2 ;
    for (n = 3; n <= max_p; n += 2)
        if (prost_3(n)) ++broj_p;

    if (prost_3(INT_MAX)) ++broj_p;

    t2 = dsecnd( );    /* Kraj mjerenja vremena. */
    time = t2 - t1;

    printf(" Broj prostih brojeva <= INT_MAX = %d je %d\n",
           INT_MAX, broj_p);
    printf("   Ukupno vrijeme = %11.6f s\n", time);

    return 0;
}
