#include <stdio.h>

/****************************************************************************/

  /* Insertion sort algoritam (sortiranje umetanjem).
     Varijanta iz Wirth (A+DS=P, str. 62).
     Sortirani dio je spijeda. Binarno trazenje, onda pomaci.
     Ako je novi element najveci, ima (nepotrebna) kopiranja.
  */

/****************************************************************************/

  /* Uzlazno sortiranje (sortiranim) umetanjem.
     Sortirani dio je sprijeda.
     Koristi binarno trazenje = binaryinsertion (Wirth, A+DS=P, str. 62). */

void insertion_sort(int x[], int n)
{
    int i, j, l, d, m, temp;

    for (i = 1; i < n; ++i) {

        temp = x[i];

            /* Cijeli skup R (ili Z) podijeljen je (uzlazno poredanim) tockama
               x[0], ..., x[i - 1] na i (zdesna) poluotvorenih intervala,
               oblika
                   I_0 = < -inf, x[0] >,      tj., kao da je x[-1] = -inf,
                   I_k = [ x[k - 1], x[k] >,  za k = 1, ..., i - 1,
                   I_i = [ x[i - 1], +inf >,  tj., kao da je x[i] = +inf.
               Uociti da neki od unutarnjih intervala smiju biti prazni.

               Trazimo indeks k, u granicama 0 <= k <= i, takav da je
               temp u intervalu I_k, tj. vrijedi x[k - 1] <= temp < x[k],
               uz OZNAKE x[-1] = -inf, x[i] = +inf (zaboravimo da je originalno
               temp = x[i]), s tim da provjere temp >= x[-1] i temp < x[i]
               nikad NECEMO raditi.

               Koristimo binarno trazenje s dva indeksa l, d, koji opisuju
               preostali dozvoljeni raspon indeksa k, za interval I_k u kojem
               se (jos) moze nalaziti temp:
                   l <= k <= d + 1  (namjerno nije d na gornjoj granici).
               Ideja je da na pocetku inicijaliziramo l = 0, d = i - 1,
               tako da je inicijalno 0 <= k <= i (kako i treba).
               Precizna znacenja indeksa l i d (u svakom trenutku) su:
                   x[l - 1] <= temp, cim je l >= 0,
                   temp < x[d + 1] , cim je d <= i - 1.

               Na pocetku algoritma, "znamo" da je sigurno
                   -inf = x[-1] <= temp,  za l = 0,
                   temp < x[i]  = +inf ,  za d = i - 1,
               pa te nejednakosti NE treba provjeravati (jer nema x[-1] i x[i]
               u smislu tocaka u R ili Z - to su samo pomocne oznake).
               Upravo zato l i d imaju navedena znacenja!

               U nastavku algoritma, nakon odgovarajucih usporedbi, mijenjamo
               indekse l i/ili d, tako da i dalje vrijedi
                   x[l - 1] <= temp, cim je l > 0,
                                     (inicijalno vrijedi za l = 0)
                   temp < x[d + 1] , cim je d < i - 1,
                                     (inicijalno vrijedi za d = i - 1).
               U binarnom trazenju usporedjujemo temp i x[m], za
               m = (l + d) / 2.
               Ako je temp < x[m], onda je d = m - 1. U suprotnom, tj.
               ako je temp >= x[m], onda je l = m + 1.

               Zato sto se trazeni indeks k nalazi u rasponu l <= k <= d + 1,
               algoritam provodimo sve dok je l <= d (postoje jos bar dva
               moguca intervala u kojima temp moze biti).
                 Algoritam staje kad je l > d i tada mora biti l = d + 1,
               a trazeni indeks k je bas jednak l.
            */

        l = 0;
        d = i - 1;
        while (l <= d) {
            m = (l + d) / 2;
            if (temp < x[m])
                d = m - 1;
            else    /* x[m] <= temp. */
                l = m + 1;
        }

            /* Pomak desnog dijela sortiranog polja za jedno mjesto udesno
               x[l], ..., x[i - 1] --> x[l + 1], ..., x[i]. */
        for (j = i; j > l; --j)
            x[j] = x[j - 1];

        x[l] = temp;    /* Moze i x[j] = temp. */

    }    /* Kraj for po i. */

    return;
}

/****************************************************************************/

int main(void)
{
    int i, n;
    int x[] = {42, 12, 55, 94, 18, 44, 67};

    n = 7;
    insertion_sort(x, n);

    printf("\n Sortirano polje x:\n");
    for (i = 0; i < n; ++i) {
        printf(" x[%d] = %d\n", i, x[i]);
    }

    return 0;
}
